#
# Copyright 2008, Moris Papasmadov
#  		  morisdov@yahoo.com
#		  http://morisdov.googlepages.com
#
# License: 
#		Free for self use if you downloaded directly from http://morisdov.googlepages.com.
#		Permission to sell or redistribute or repackage this software 
#		should be specifically requested from the author.
#		above copyright notice should appear in all copies.
#
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#
#
# Author:	Moris Papasmadov
#
# Name :        proper_animation.rb
#
# Description:  Make SketchUp models live by integrating movements to preset positions
#		to demonstrate models in real life action like closed and opened doors, windows
#		progress of a construction site or assembly procedures... etc
#		
# Usage:	right_mouse_click -> Transformations -> Set Position X / Get Position X
#		 
#		1)Add animation scenes to your model (with changed views if desired)
#		2)Menu -> Plugins -> Start animation observer
#		3)select the first scene tab to move entities to saved Position 1
#		  select the second scene tab to move entities to saved Position 2 
#		  select the X scene tab to move entities to saved Position X etc
#		4)Menu -> Plugins -> Stop animation observer
#
# 		In SketchUp - open your model, switch to "Select" mode
#		Left_Mouse_Click on one entity or entities to select them, then
#		right_mouse_click -> Transformations -> Set Position X
#		to remember current positions of selected entities - Groups and Components
#		(edges and faces dont have transformation data)
#
#		To restore previously "saved" positions, again select desired entities, then 
#		right_mouse_click -> Transformations -> Get Position X.
#		If "Get position X" is grayed out, not all entities in your current selection have
#		saved position corresponding to X Position.
#		Remove from your selection those entities that do not have a saved position
#		corresponding to Position X.
#
#		Stop and Start again "Animation Observer" to reflect new set positions !
#
# Installation: copy file "proper_animation.rb"  into "Plugins" folder under SketchUp installation
#		restart SketchUp and note a new right_mouse_click context menu item
#		at the bottom, named "Transformations"
#		note two new menu items under "plugins" named "start animation observer" and
#		and "Stop animation observer".
#		
#
# Known issues: Menu -> view -> animation -> play/stop buttons, stop the movement of
#		entities somewhere half way between previously saved positions -
#		click manually on some scene tabs to move items manually to preset locations
#
#		Menu->file->export->animation - does not support the animation observer :-(
#		I will try to find workaround
# 
#		Stop and Start again "Animation Observer" to reflect new set positions !
#
#
# Ruby code:	My original version of this plugin does not work with attribute dictionaries
#		it gets the positions data outside of SketchUp - so the code below seems messy 
#		
# Version:	1.04, 27-July-2008
#
#		1.04 added option to control each scenes transition time
#		1.03 no need to restart "Animation Observer" to reflect new/deleted scenes
#		1.02 minor speed improvements, unlimited position options
#		1.01 fixed a shrink illusion during rotation
#		1.00 first release
#		come back in a few weeks to get a better debugged version
#
#		http://morisdov.googlepages.com
#		
#		
#		
#
module Morisdov
#--------------------------------------------------------------------------------
def Morisdov.transf_set(dict_key)

  Sketchup.active_model.selection.each {|e|
    dict_name="positiondata"
	begin
	dict_value = e.transformation.to_a
	e.attribute_dictionary(dict_name, true)
	e.set_attribute(dict_name, dict_key, dict_value)
	p dict_value.to_a
	rescue
	end
  }
end  # def
#--------------------------------------------------------------------------------
def Morisdov.transf_get(dict_key)

  model=Sketchup.active_model					
  ents=model.active_entities
  @ents=[]									
  model.selection.each {|e| @ents.push(e)}			
  @ents.freeze
  model.selection.clear

  @e_dict_value = []
  @n =	0.1
  dict_name = "positiondata"

@ents.each {|e|
	  dicts = e.attribute_dictionaries
	  dict_value = []
	  if( (dicts != nil) and e.get_attribute(dict_name, dict_key) != nil )
	    dict_value = e.get_attribute(dict_name, dict_key).to_a
	    @e_dict_value.push(dict_value)
	    else
	    p "No transf set for this  " + e.typename   # catch
	    @ents.delete(e)
	  end # if
	  }
  
#  @my_dialog.show
#  @my_dialog.execute_script("setCursor('wait')")
  var = 0.1
  @timer =  UI.start_timer(var, true) { Morisdov.transf_step }
  Sketchup.active_model.start_operation "Transformation"
  p "Transformation timer started"

end   # def
#--------------------------------------------------------------------------------
def Morisdov.transf_step()

if @n<1
    @ents.each {|e|
	dict_value = @e_dict_value[@ents.index(e)]
	if dict_value != []
	current_location = e.transformation.to_a
	    
	#temp_location = current_location 
	temp_location = (Geom::Transformation.interpolate current_location, dict_value, @n).to_a
	    #p current_location
	    #p temp_location.to_a
	    #p dict_value
	    Sketchup.active_model.active_entities.transform_entities(e.transformation.inverse,e)
	    Sketchup.active_model.active_entities.transform_entities(temp_location,e)
	end # if
    }
    @n += 0.1

else

    UI.stop_timer @timer
    p "Transformation timer stopped"
    Sketchup::set_status_text " Transformation completed"

    @ents.each {|e|
	dict_value = @e_dict_value[@ents.index(e)]
	    Sketchup.active_model.active_entities.transform_entities(e.transformation.inverse,e)
	    Sketchup.active_model.active_entities.transform_entities(dict_value,e)
	    p dict_value.to_a
	Sketchup.active_model.selection.add(e)
    }
    Sketchup.active_model.commit_operation
    @n += 0.1
#    @my_dialog.execute_script("setCursor('default')")
#    @my_dialog.close
end # if
end # def
#--------------------------------------------------------------------------------

#--------------------------------------------------------------------------------
class FrameChangeObserver
#--------------------------------------------------------------------------------
def initialize()
	prepare_animated_arrays()
end
#--------------------------------------------------------------------------------
def prepare_animated_arrays()

	@@ps = []
	@@ps.clear
	Sketchup.active_model.pages.each {|page| @@ps.push(page) } 
	@@ps.freeze
	#page = Sketchup.active_model.pages.selected_page 
	#@@ps.each {|page| p page}

    dict_name="positiondata"
    @@en_vals = []
    @@en = []
  
	1.upto(Sketchup.active_model.pages.count) do |x|			
	  en_vals = {} # hash
	  en = []

	    Sketchup.active_model.active_entities.each {|e|
		  dict_value = []
		  dicts = e.attribute_dictionaries

		  if (dicts != nil) 
			  if e.get_attribute(dict_name, "transf" + x.to_s) != nil
			    dict_value = e.get_attribute(dict_name, "transf" + x.to_s).to_a
			    en_vals[e] = dict_value
			    en.push(e)
			  end
		  end # if
	    }
	@@en_vals.push(en_vals)
	@@en.push(en)
	end # do
  
  p "Prepared animated arrays " + @@en.length.to_s
end
#--------------------------------------------------------------------------------
def frameChange(fromPage, toPage, parameter)

	#p @ps.index(fromPage), @ps.index(toPage), parameter
	#p @@ps.index(toPage).to_s + " : " + parameter.to_s


	ents = @@en[@@ps.index(toPage)]
	e_vals = @@en_vals[@@ps.index(toPage)]
	#p ents


	if ents != nil
		if parameter  < 1
		    ents.each {|e|
			dict_value = e_vals[e]
			#p dict_value 
			if dict_value != []
			    current_location = e.transformation.to_a
			    #temp_location = current_location 
			    temp_location = Geom::Transformation.interpolate current_location, dict_value, parameter
			    #p current_location
			    #p temp_location 
			    #p dict_value
			    Sketchup.active_model.active_entities.transform_entities(e.transformation.inverse,e)
			    Sketchup.active_model.active_entities.transform_entities(temp_location,e)
			end # if
		    }
		else
		    ents.each {|e|
			dict_value = e_vals[e]
			    Sketchup.active_model.active_entities.transform_entities(e.transformation.inverse,e)
			    Sketchup.active_model.active_entities.transform_entities(dict_value,e)
			    #p dict_value
		    }
		end # if
	else
	p "ents = nil"
	end # if

end # method
#--------------------------------------------------------------------------------
end # class
#--------------------------------------------------------------------------------

#--------------------------------------------------------------------------------
class MyPagesObserver < FrameChangeObserver
 
	def onElementAdded(el,what)
		prepare_animated_arrays()
	end

	def onElementRemoved(el,what)
		prepare_animated_arrays()
	end
#--------------------------------------------------------------------------------
end # class
#--------------------------------------------------------------------------------






#--------------------------------------------------------------------------------
def Morisdov.testObserver(str)
		return 1 if (str == "add" and @hndl != nil)
		return 1 if (str == "remove" and  @hndl == nil)
	return 0 
end
#--------------------------------------------------------------------------------
if( not @anima_menu_loaded )
   menu = UI.menu("Plugins")
   UI.menu("Plugins").add_separator

   subhndl=menu.add_item("Start Animation Observer"){
		
		@hndl = Sketchup::Pages.add_frame_change_observer FrameChangeObserver.new 
		p "Started Animation Observer " + @hndl.to_s
		@hndl2 = Sketchup.active_model.pages.add_observer MyPagesObserver.new 
		}  
		menu.set_validation_proc(subhndl)  { Morisdov.testObserver("add") }	
   
   subhndl=menu.add_item("Stop Animation Observer"){
		Sketchup::Pages.remove_frame_change_observer(@hndl) 
		p "Stopped Animation Observer " + @hndl.to_s
		@hndl = nil 
		Sketchup.active_model.pages.remove_observer(@hndl2)
		}  
		menu.set_validation_proc(subhndl)  { Morisdov.testObserver("remove") }	


   @anima_menu_loaded = true
end
#--------------------------------------------------------------------------------



#--------------------------------------------------------------------------------
def Morisdov.scenes_transition_times()
	ps = []
	prompts = []
	values = []

	Sketchup.active_model.pages.each {|page| ps.push(page);prompts.push(page.name) }
	#p ps
	#p prompts
	ps.each {|page| values.push(page.transition_time) }	
	#p values

	results = inputbox prompts, values, "Scenes Transition Times"
	#p results

	if results
		0.upto(ps.length-1) do |x|			
		   p results[x]
		   ps[x].transition_time = results[x]
		end # do
	end
end
#--------------------------------------------------------------------------------
if( not @scenes_transition_times_loaded )
   menu = UI.menu("Plugins")

   UI.menu("Plugins").add_separator
   menu.add_item("Scenes Transition times") { Morisdov.scenes_transition_times() }

   @scenes_transition_times_loaded = true
end
#--------------------------------------------------------------------------------


#--------------------------------------------------------------------------------
def Morisdov.testTransfAttr(dict_key)

	Sketchup.active_model.selection.each {|e| 
		return 1 if (e.attribute_dictionaries == nil)
		return 1 if (e.get_attribute("positiondata",dict_key) == nil)
	}
	return 0 
end
#--------------------------------------------------------------------------------
def Morisdov.addMenuItem(submenu,x)
	submenu.add_item("Set Position - " + x.to_s) { Morisdov.transf_set("transf" + x.to_s) }
	subhndl=submenu.add_item("Get Position - " + x.to_s) { Morisdov.transf_get("transf" + x.to_s) }
		submenu.set_validation_proc(subhndl)  { Morisdov.testTransfAttr("transf" + x.to_s) }	
	submenu.add_separator
	return subhndl
end
#--------------------------------------------------------------------------------
if(not @trans_menu_loaded)
	# Create the WebDialog instance
	#html_path = Sketchup.find_support_file "trans.htm" ,"Plugins\\Examples"
	#@my_dialog = UI::WebDialog.new
	#@my_dialog.set_position 9999, 9999
	#@my_dialog.set_file(html_path)

	@number_of_positions = 7
	UI.add_context_menu_handler do |menu|
		menu.add_separator
		submenu=menu.add_submenu("Transformations")
			1.upto(@number_of_positions) do |x|			
				Morisdov.addMenuItem(submenu,x)
			end # do
		submenu.add_item("Add Position - " + (@number_of_positions+1).to_s) { @number_of_positions += 1}
	end  # do
	@trans_menu_loaded = true
end  # if
#--------------------------------------------------------------------------------

end # module